library(Matrix)
library(nadiv)
library(ggplot2)

GenA <- function(male, female) {
  if (nargs() == 1) {
    stop("require male and female entries")
  }
  if (length(male) != length(female)) {
    stop("length of male and female differ")
  }
  male[is.na(male)] <- 0  # convert all NA to zeros
  female[is.na(female)] <- 0  # convert all NA to zeros
  n <- length(male)
  N <- n + 1
  A <- matrix(0, ncol = N, nrow = N)
  male <- (male == 0) * (N) + male
  female <- (female == 0) * N + female
  for (i in 1:n) {
    A[i, i] <- 1 + A[male[i], female[i]]/2
    for (j in (i + 1):n) {
      if (j > n)
        break
      A[i, j] <- (A[i, male[j]] + A[i, female[j]])/2
      A[j, i] <- A[i, j]
    }
  }
  A <- as(A, "sparseMatrix")
  return(A[1:n, 1:n])
}



MultipleDesigns <- function(DesN, blocks, Treatments, rb,cb, Tr, Tc,Amat=FALSE,
                            criteria="A", h2, rhox,rhoy, s20,irregular=FALSE) {
  matrix0 <- list()
  initialValues1 <- c()
  initialValues2 <- c()
  for (i in 1:DesN) {
    print(sprintf("generating initial design: %d", i, "complete\n",
                  sep = ""))
    flush.console()
    matrix0[[i]] <- rcbd(blocks, Treatments, rb,cb, Tr, Tc, irregular)
    initialValues1[i] <- VarCov.rcbd(matdf = matrix0[[i]],rhox,rhoy,h2,s20,Tr,Tc,criteria,Amat,irregular)[[1]]
    a <- which.min(initialValues1)
    newmatdfA <- matrix0[a][[1]]
    min_initialValues1 <- initialValues1[a][[1]]
  }
  return(list(newmatdf = newmatdfA, trace0 = initialValues1,
              min_value = min_initialValues1, meanA = mean(initialValues1)))
}


NewValue.rcbd <- function(matdf, criteria, Rinv, Ginv, K) {
  Z <- Matrix::sparse.model.matrix(~as.factor(matdf[, "Treatments"]) - 1)
  Z <- as.matrix(Z)
  temp0 <- t(Z) %*% Rinv %*% Z + Ginv - t(Z) %*% K %*% Z
  C22 <- solve(temp0)
  if (criteria == "A") {
    traceI <- sum(Matrix::diag(C22))
    return(traceI)
  }
  if (criteria == "D") {
    doptimI <- log(Matrix::det(C22))
    return(doptimI)
  }
}

Optimize.rcbd<- function(matdf,n,traceI,criteria,Rinv,Ginv,K) {
  newmatdf <- matdf
  trace <- traceI
  mat <- NULL
  mat <- rbind(mat, c(value = trace, iterations = 0))
  Design_best <- newmatdf
  Des <- list()
  TRACE <- c()
  newmatdf <- SwapPair(matdf = matdf)
  for (i in 2:n) {
    newmatdf <- SwapPair(matdf = newmatdf)
    TRACE[i] <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
    Des[[i]] <- newmatdf
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) < trace) {
      print(sprintf("Swapping within blocks: %d", i, "complete\n",
                    sep = ""))
      Design_best <- Des[[i]] <- newmatdf
      Design_best <- newmatdf
      trace <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
      mat <- rbind(mat, c(trace = trace, iterations = i))
    }
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace & nrow(mat) <= 1) {
      newmatdf <- matdf
      Des[[i]] <- matdf
      Design_best <- matdf
    }
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace & nrow(mat) > 1) {
      newmatdf <- Des[[length(Des) - 1]]
      Des[[i]] <- newmatdf
      Design_best <- newmatdf
    }
  }
  ODE = (((mat[1,"value"]) - (mat[nrow(mat),"value"]))/(mat[1,"value"]))*100
  print(sprintf("ODE due to swapping pairs of treatments within blocks is: %f", ODE, "complete\n",
                sep = ""))
  list(TRACE = c(as.vector(mat[1, "value"]), TRACE[!is.na(TRACE)]), mat = mat,
       Design_best = Design_best)
}

rcbd<- function(blocks, Treatments,rb,cb, Tr, Tc, irregular=FALSE) {
  genot0 <- as.numeric(as.factor(Treatments))
  n <- length(Treatments)
  Treatments <- c(replicate(blocks, sample(genot0, n, replace = FALSE)))
  Reps <- rep(1:blocks, each = length(unique(Treatments)))
  if(blocks==1){
    cord <- cbind(Row=rep(1:Tr,each=Tc), Col=rep(1:Tc,Tr))
    matdf <- cbind(cord, Reps=c(rep(1,n)),Treatments)
    row.names(matdf)<-NULL
  }
  if(cb == Tc & Tr > rb & irregular==FALSE){
    cord <- cbind(Row=rep(1:Tr,each=cb), Col=rep(1:Tc,rb))
    matdf <- cbind(cord,Reps, Treatments)
    row.names(matdf)<-NULL
  }
  if(rb == Tr & Tc > cb & irregular==FALSE){
    Row=rep(rep(1:Tr,each=cb),Tc/cb)
    Col <- rep(split(1:Tc, cut(seq_along(1:Tc), blocks, labels = FALSE)),each=Tr)
    Col <- unlist(Col)
    cord <- cbind(Row,Col)
    matdf <- cbind(cord,Reps, Treatments)
    row.names(matdf)<-NULL
  }
  if(Tr > rb & Tc > cb & irregular==FALSE){
    Row=rep(rep(1:Tr,each=cb),Tc/cb)
    Col <- rep(split(1:Tc, cut(seq_along(1:Tc), Tc/cb, labels = FALSE)),each=Tr)
    Col <- unlist(Col)
    cord <- cbind(Row,Col)
    matdf <- cbind(cord,Reps, Treatments)
    row.names(matdf)<-NULL
  }
  if(irregular==TRUE){
    cord <-cbind(Row=Row,Col=Col)
    matdf <- cbind(cord, Reps, Treatments)
    row.names(matdf)<-NULL
  }
  matdf[order(matdf[,"Row"],matdf[,"Col"]),]
}


SwapPair <- function(matdf) {
  mat <- data.frame(matdf, row.names = NULL)
  b1 <- sample(mat$Reps, 1, replace = TRUE)
  gg1 <- mat$Treatments[mat$Reps == b1]
  g1 <- sample(gg1, 2)
  temp <- gg1
  temp[gg1 == g1[1]] <- g1[2]
  temp[gg1 == g1[2]] <- g1[1]
  mat$Treatments[mat$Reps == b1] <- temp
  mat[order(mat[,"Row"],mat[,"Col"]),]
}


VarCov.rcbd <- function(matdf, rhox, rhoy, h2, s20, Tr, Tc, criteria="A", Amat=FALSE,irregular=FALSE) {
  if(nrow(matdf)==length(unique(matdf[,"Treatments"]))){
    X <- as.matrix(matdf[, "Reps"])
  }
  if(nrow(matdf) > length(unique(matdf[,"Treatments"]))){
    X <- Matrix::sparse.model.matrix(~as.factor(matdf[, "Reps"])-1)
  }
  
  s2e <- (1 - s20) * (1 - h2)
  stopifnot(s2e > 0)
  
  m = length(unique(matdf[,"Treatments"]))
  if(is.matrix(Amat)){
    G <- h2 * as.matrix(Amat)
    Ginv <- round(chol2inv(chol(as.matrix(G))),7)
    Ginv <- as(Ginv, "sparseMatrix")
  }
  else{
    Ginv <- round((1/h2) * Matrix::Diagonal(m),7)
    Ginv <- as(Ginv, "sparseMatrix")
  }
  
  Z<- Matrix::sparse.model.matrix(~as.factor(matdf[, "Treatments"]) - 1)
  
  # calculating R and its inverse for spatial analysis
  bb <- length(unique(matdf[,"Reps"]))
  matdf <- matdf[order(matdf[,"Row"],matdf[,"Col"]),]
  if(irregular==TRUE){
    R <- Matrix::Diagonal(nrow(matdf))
    for(i in 1:(nrow(matdf)-1)) {
      x1  <- matdf[,"Col"][i]
      y1  <- matdf[,"Row"][i]
      for (j in (i+1):nrow(matdf)){
        x2 <- matdf[,"Col"][j]
        y2  <- matdf[,"Row"][j]
        R[i,j]<-(rhox^abs(x2 -x1))*(rhoy^abs(y2 -y1))
      }
    }
    R = as.matrix(round(s2e*R,7))
    R[lower.tri(R)] <- t(R)[lower.tri(R)]
    R <- as(R, "sparseMatrix")
    Rinv <- round(chol2inv(chol(R)),7)
    Rinv <- as(Rinv, "sparseMatrix")
  }
  
  if(irregular==FALSE){
    sigx <- Matrix::Diagonal(Tc)
    sigx <- rhox^abs(row(sigx) - col(sigx))
    sigy <- Matrix::Diagonal(Tr)
    sigy <- rhoy^abs(row(sigy) - col(sigy))
    R <- round(s2e * kronecker(sigy, sigx),7)
    R <- as(R, "sparseMatrix")
    Rinv <- round(chol2inv(chol(R)),7)
    Rinv <- as(Rinv, "sparseMatrix")
  }
  
  C11 <- Matrix::crossprod(as.matrix(X), as.matrix(Rinv)) %*% as.matrix(X)
  C11inv <- solve(C11)
  k1 <- Rinv %*% as.matrix(X)
  k2 <- Matrix::tcrossprod(as.matrix(C11inv), as.matrix(X))
  k3 <- k2 %*% Rinv
  K <- k1 %*% k3
  K <- as(K, "sparseMatrix")
  temp0 <- Matrix::crossprod(Z, Rinv) %*% Z + Ginv - Matrix::crossprod(Z, K) %*% Z
  C22 <- solve(temp0)
  C22  <- as(C22, "sparseMatrix")
  Ginv = round(Ginv,7)
  Rinv = Matrix::drop0(round(Rinv,7))
  K = round(K,7)
  C22 = round(C22,7)
  if (criteria == "A") {
    return(c(traceI = sum(Matrix::diag(C22)), Ginv = Ginv, Rinv = Rinv, K=K))
  }
  if (criteria == "D") {
    deTm = Matrix::det(C22)
    return(c(doptimI = log(deTm), Ginv = Ginv, Rinv = Rinv, K=K))
  }
}

Error.Surfaces<-function(Tr,Tc,rhox,rhoy,s20,h2){
  # calculation of base V(N*N) matrix with AR(1)*AR(1) covariance model
  e1=rnorm(Tr*Tc);e2=rnorm(Tr*Tc)   # e1,e2: independent vectors of errors
  N = Tr*Tc
  V = diag(N)
  sigx <- Matrix::Diagonal(Tc)
  sigx <- rhox^ abs(row(sigx) - col(sigx))
  sigy <- Matrix::Diagonal(Tr)
  sigy <- rhoy ^ abs(row(sigy) - col(sigy))
  s2e <- (1-s20)*(1-h2)
  V<- s2e * kronecker(sigy, sigx)  # takes 0.01 second
  L = chol(V)
  PAT = t(L)%*%e1
  PAT = (PAT - mean(PAT))/sd(PAT) # standardized patches
  ZST = sqrt(1-h2)*(PAT*sqrt(1-s20)+e2*sqrt(s20))
  ZST
}

ReplaceTrts <- function(matdf,Treatments,max.u,min.u)
{
  freq = as.numeric(table(matdf[,"Treatments"]))
  Dif = max.u - freq
  genot <- as.numeric(as.factor(Treatments))
  datam = cbind(genot,min.u,max.u,freq,Dif)
  u.extra = datam[freq < max.u,"genot"]
  numT <- c(Dif[Dif != 0])
  extraG =rep(u.extra,numT)
  sam <- sample(genot[freq > min.u],1,replace=TRUE)
  sam <- which(matdf[,"Treatments"]==sam)[1]
  Lg <- sample(extraG[extraG!=sam],1,replace=TRUE)
  Lg <- which(matdf[,"Treatments"]==Lg)[1]
  matdf[,"Treatments"][sam] <- matdf[,"Treatments"][Lg]
  freq = as.numeric(table(matdf[,"Treatments"]))
  datam = cbind(genot,min.u,max.u,freq)
  list(matdf=matdf,datam=datam, sumFreq = sum(freq),changed_index_dataframe = sam)
}

SwapAnyPair <- function(matdf) {
  mat0 <- matdf[,"Treatments"]
  sam <- sample(mat0,2, replace = TRUE)
  mat0[c(sam[1],sam[2])] <- mat0[c(sam[2],sam[1])]
  matdf[,"Treatments"] <- mat0
  matdf[order(matdf[,"Row"],matdf[,"Col"]),]
}

Neighbor_rcbd<-function(matdf,Amat)
{
  bl<-sample(matdf[,"Reps"],1)
  temp1<- matdf[matdf[, "Reps"] == bl,]
  rb <-length(unique(temp1[,"Row"]))
  cb <-length(unique(temp1[,"Col"]))
  mat<-matrix(c(temp1[,"Treatments"]),nrow=rb,ncol=cb,byrow=TRUE)
  x<-sample(temp1[,"Treatments"],1)
  m2<-cbind(NA,rbind(NA,mat,NA),NA)
  cord <- expand.grid(Row = 1:rb, Col = 1:cb)
  ret<-c()
  for(i in 1:-1)
    for(j in 1:-1)
      if(i!=0 || j !=0)
        ret<-rbind(ret,m2[cord$Row+i+1+nrow(m2)*(cord$Col+j)])
  neigh<-ret[,which(mat==x)]
  neigh<-c(x,neigh[!is.na(neigh)])  # add the genotype plus its neighbors
  test<-t(combn(neigh,2))
  temp<-cbind(test[,1],test[,2],Amat[test])
  swapsData<-subset(temp, temp[,3] >= 0.25)
  samp1 <- setdiff(as.vector(temp1[,"Treatments"]),c(x,as.vector(temp[,2])))
  if(nrow(swapsData)!=0 && length(samp1) !=0)
  {
    val1<-c()
    val2<-c()
    for(i in 1:nrow(swapsData))
    {
      val1[i]<-swapsData[i,1]
      val2[i]<-sample(samp1,1)
      matdf<-Swap_Specific(matdf,g1=val1[i],g2=val2[i],bl=bl)
    }
    return(matdf[order(matdf[,"Row"],matdf[,"Col"]),])
  }
  if(nrow(swapsData)!=0 && length(samp1)==0)
  {
    val1<-c()
    val2<-c()
    for(i in 1:nrow(swapsData))
    {
      val1[i]=swapsData[i,1]
      val2[i]=swapsData[i,2]
      matdf<-Swap_Specific(matdf,g1=val1[i],g2=val2[i],bl=bl)
    }
    return(matdf[order(matdf[,"Row"],matdf[,"Col"]),])
  }
  if(nrow(swapsData)==0) return(SwapPair(matdf))
}


Optimize_GNN_rcbd<- function(matdf,n,traceI,criteria,Amat, Rinv, Ginv, K)
{
  newmatdf <- matdf
  trace <- traceI
  mat <- NULL
  mat <- rbind(mat, c(value = trace, iterations = 0))
  Design_best <- newmatdf
  Des <- list()
  TRACE <- c()
  newmatdf <- Neighbor_rcbd(matdf,Amat)
  for (i in 2:n) {
    newmatdf <- Neighbor_rcbd(matdf,Amat)
    TRACE[i] <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
    Des[[i]] <- newmatdf
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) < trace) {
      print(sprintf("Swapping treatments: %d", i, "complete\n",
                    sep = ""))
      Design_best <- Des[[i]] <- newmatdf
      Design_best <- newmatdf
      trace <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
      mat <- rbind(mat, c(trace = trace, iterations = i))
    }
    if(NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace && nrow(mat)<=1){
      newmatdf <- matdf
      Des[[i]] <- matdf
      Design_best <- matdf
    }
    if(NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace && nrow(mat)>1){
      newmatdf <- Des[[length(Des) - 1]]
      Des[[i]] <- newmatdf
      Design_best <- newmatdf
    }
  }
  ODE = (((mat[1,"value"]) - (mat[nrow(mat),"value"]))/(mat[1,"value"]))*100
  print(sprintf("ODE due to applying GNN procedure: %f", ODE, "complete\n",
                sep = ""))
  list(TRACE = c(as.vector(mat[1, "value"]), TRACE[!is.na(TRACE)]), mat = mat,
       Design_best = Design_best)
}

Optimize_SimAnn_rcbd<- function(matdf,n,traceI,criteria,Rinv,Ginv,K) {
  newmatdf <- matdf
  trace <- traceI
  mat <- NULL
  mat <- rbind(mat, c(value = trace, iterations = 0))
  Design_best <- newmatdf
  Des <- list()
  TRACE <- c()
  newmatdf <- SwapPair(matdf = matdf)
  for (i in 2:n) {
    newmatdf <- SwapPair(matdf = newmatdf)
    TRACE[i] <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
    Des[[i]] <- newmatdf
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) < trace) {
      print(sprintf("Swapping within blocks: %d", i, "complete\n",
                    sep = ""))
      Design_best <- Des[[i]] <- newmatdf
      Design_best <- newmatdf
      trace <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
      mat <- rbind(mat, c(trace = trace, iterations = i))
    }
    Temp<-c()
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace)
    {
      dif <- setdiff(NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K),trace)
      Temp[i] <- 1/i
      accept  = exp(-dif/Temp[i])
      u = runif(1)
      if (u < accept){
        Design_best <- Des[[i]] <- newmatdf
        Design_best <- newmatdf
        trace <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
      }
      if (u > accept & nrow(mat) <= 1) {
        newmatdf <- matdf
        Des[[i]] <- matdf
        Design_best <- matdf
      }
      if (u > accept & nrow(mat) > 1) {
        newmatdf <- Des[[length(Des) - 1]]
        Des[[i]] <- newmatdf
        Design_best <- newmatdf
      }
    }
  }
  ODE = (((mat[1,"value"]) - (mat[nrow(mat),"value"]))/(mat[1,"value"]))*100
  print(sprintf("ODE due to simulated annealing is: %f", ODE, "complete\n",
                sep = ""))
  list(TRACE = c(as.vector(mat[1, "value"]), TRACE[!is.na(TRACE)]), mat = mat,
       Design_best = Design_best)
}


OptimizeGreedy.rcbd<- function(matdf,n,traceI,criteria,gsize,Rinv,Ginv,K) {
  newmatdf <- matdf
  trace <- traceI
  mat <- NULL
  mat <- rbind(mat, c(value = trace, iterations = 0))
  Design_best <- newmatdf
  Des <- list()
  TRACE <- c()
  newmatdf <- SwapGreedy(matdf = matdf,gsize = gsize)
  for (i in 2:n) {
    newmatdf <- SwapGreedy(matdf = newmatdf,gsize = gsize)
    TRACE[i] <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
    Des[[i]] <- newmatdf
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) < trace) {
      print(sprintf("Swapping greedly within blocks: %d", i, "complete\n",
                    sep = ""))
      Design_best <- Des[[i]] <- newmatdf
      Design_best <- newmatdf
      trace <- NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K)
      mat <- rbind(mat, c(trace = trace, iterations = i))
    }
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace & nrow(mat) <= 1) {
      newmatdf <- matdf
      Des[[i]] <- matdf
      Design_best <- matdf
    }
    if (NewValue.rcbd(matdf=newmatdf, criteria, Rinv, Ginv, K) > trace & nrow(mat) > 1) {
      newmatdf <- Des[[length(Des) - 1]]
      Des[[i]] <- newmatdf
      Design_best <- newmatdf
    }
  }
  ODE = (((mat[1,"value"]) - (mat[nrow(mat),"value"]))/(mat[1,"value"]))*100
  print(sprintf("ODE due to greedly swapping pairs of treatments within blocks is: %f", ODE, "complete\n",
                sep = ""))
  list(TRACE = c(as.vector(mat[1, "value"]), TRACE[!is.na(TRACE)]), mat = mat,
       Design_best = Design_best)
}


Swap_Specific <- function(matdf,g1,g2,bl) {
  mat <- data.frame(matdf, row.names = NULL)
  gg1 <- mat$Treatments[mat$Reps == bl]
  temp <- gg1
  temp[gg1 == g1] <- g2
  temp[gg1 == g2] <- g1
  mat$Treatments[mat$Reps == bl] <- temp
  mat[order(mat[,"Row"],mat[,"Col"]),]
}


SwapGreedy <- function(matdf,gsize) {
  stopifnot(gsize %% 2 == 0)
  mat <- data.frame(matdf, row.names = NULL)
  b1 <- sample(mat$Reps, 1, replace = TRUE)
  gg1 <- mat$Treatments[mat$Reps == b1]
  g1 <- sample(gg1, gsize)
  temp <- gg1
  for (i in seq(1,length(g1),2))
  {
    temp[gg1 == g1[i]] <- g1[i+1]
    temp[gg1 == g1[i+1]] <- g1[i]
  }
  mat$Treatments[mat$Reps == b1] <- temp
  mat[order(mat[,"Row"],mat[,"Col"]),]
}


## Produce a plot of designs

plotdesign <- function(matdf,rb,cb,Tr,Tc,text.size=4,
                       titlesize=17,axislabsize=15,
                       axistextsize=12,
                       maintitle="Initial Design"){
  if(Tr > rb & Tc > cb){
    matdf = as.data.frame(matdf)
    P = ggplot2::ggplot(data=matdf) + 
      geom_text(aes(x=Col,y=as.factor(Row),
                    label=Treatments),
                size=text.size) +
      scale_y_discrete("Row coordinates",
                       expand = c(0.055,0.055)) +
      scale_x_continuous(breaks=seq(min(matdf$Col),
                                    max(matdf$Col),1),
                         expand = c(0.05,0.05),
                         "Column coordinates") +
      scale_color_discrete("Block") +
      ggtitle(label = maintitle) +
      geom_vline(xintercept=seq(cb+0.5,Tc,cb),
                 linetype="solid", size=1,colour="darkred") +
      geom_hline(yintercept=seq(rb+0.5,Tr,rb),
                 linetype="solid", size=1,colour="darkred") +
      theme(plot.title = element_text(size=titlesize, face="bold",hjust=0.5),
            axis.text=element_text(size=axistextsize, face="bold"),
            axis.title=element_text(size=axislabsize,face="bold"),
            legend.title = element_text(face="bold"),
            legend.background = element_rect(colour = "black"),
            panel.border = element_rect(colour="darkblue",
                                        fill="NA",size = 2,linetype = "solid"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.background = element_blank(), 
            axis.line = element_line("darkblue",size = 2,linetype = "solid"),
            text = element_text(size=12,face="bold")) +  
      coord_fixed() +
      guides(colour=FALSE)
    print(P)
  }
  if(Tc > cb & Tr == rb){
    matdf = as.data.frame(matdf)
    P = ggplot2::ggplot(data=matdf) +
      geom_text(aes(x=Col, y=as.factor(Row),
                    label=Treatments), size=text.size) +
      scale_y_discrete("Row coordinates") +
      scale_x_continuous(breaks=seq(min(matdf$Col),
                                    max(matdf$Col),1), 
                         expand = c(0.05,0.05),
                         "Column coordinates") +
      scale_color_discrete("Block") +
      ggtitle(label = maintitle) +
      geom_vline(xintercept=seq(cb+0.5, Tc, cb),
                 linetype="solid", size=1,colour="darkred") +
      theme(plot.title = element_text(size=titlesize,face="bold",hjust=0.5),
            axis.text=element_text(size=axistextsize,face="bold"),
            axis.title=element_text(size=axislabsize,face="bold"),
            legend.title = element_text(face="bold"),
            legend.background = element_rect(colour = "black"),
            panel.border = element_rect(colour="darkblue",
                                        fill="NA",size = 2,linetype = "solid"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.background = element_blank(), 
            axis.line = element_line("darkblue",size = 2,linetype = "solid"),
            text = element_text(face="bold")) +  
      coord_fixed() +
      guides(colour=FALSE)
    print(P)
  }
  if(Tr > rb & Tc == cb){
    matdf = as.data.frame(matdf)
    P = ggplot2::ggplot(data=matdf)+geom_text(aes(x=Col,
                                                  y=as.factor(Row),label=Treatments), size=text.size) +
      scale_y_discrete("Row coordinates",expand = c(0.05,0.05)) +
      scale_x_continuous(breaks=seq(min(matdf$Col),
                                    max(matdf$Col),1),
                         "Column coordinates",expand = c(0.12,0.12)) +
      scale_color_discrete("Block") +
      ggtitle(label = maintitle) +
      geom_hline(yintercept=seq(rb+0.5, Tr, rb),
                 linetype="solid", size=1,colour="darkred") +
      theme(plot.title = element_text(size=titlesize,
                                      face="bold",hjust=0.5),
            axis.text=element_text(size=axistextsize,
                                   face="bold"),
            axis.title=element_text(size=axislabsize,
                                    face="bold"),
            legend.title = element_text(face="bold"),
            legend.background = element_rect(colour = "black"),
            panel.border = element_rect(colour="darkblue",
                                        fill="NA",size = 2,
                                        linetype = "solid"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.background = element_blank(), 
            axis.line = element_line("darkblue",
                                     size = 2,linetype = "solid"),
            text = element_text(face="bold")) +  
      coord_fixed() +
      guides(colour=FALSE)
    print(P)
  }
}
#####




